from argparse import ArgumentParser

import i18n

from brainframe.cli import print_utils, docker_compose, env_vars
from .utils import subcommand_parse_args, command


@command("update")
def update():
    args = _parse_args()

    install_path = env_vars.install_path.get()

    docker_compose.assert_installed(install_path)

    print_utils.translate("general.downloading-docker-compose")
    docker_compose_path = install_path / "docker-compose.yml"
    docker_compose.download(docker_compose_path, version=args.version)

    docker_compose.run(install_path, ["pull"])

    if args.noninteractive:
        restart = args.restart
    else:
        restart = print_utils.ask_yes_no("update.ask-restart")
    if restart:
        docker_compose.run(install_path, ["down"])
        docker_compose.run(install_path, ["up", "-d"])

    print()
    print_utils.translate("update.complete", color=print_utils.Color.GREEN)


def _parse_args():
    parser = ArgumentParser(
        description=i18n.t("update.description"), usage=i18n.t("update.usage")
    )

    parser.add_argument(
        "--noninteractive",
        action="store_true",
        help=i18n.t("general.noninteractive-help"),
    )

    parser.add_argument(
        "--version",
        type=str,
        default="latest",
        help=i18n.t("update.version-help"),
    )

    parser.add_argument(
        "--restart", action="store_true", help=i18n.t("update.restart-help")
    )

    return subcommand_parse_args(parser)



import os
import subprocess
import sys
from pathlib import Path
from typing import List

import distro
import i18n

from . import print_utils

BRAINFRAME_GROUP_ID = 1337
"""An arbitrary group ID value for the 'brainframe' group. We have to specify
the ID of the group manually to ensure that the host machine and the Docker
containers agree on it.
"""


def create_group(group_name: str, group_id: int):
    # Check if the group exists
    result = run(["getent", "group", group_name], exit_on_failure=False)
    if result.returncode == 0:
        print_utils.translate("install.group-exists")
        return

    # Create the group
    result = run(["groupadd", group_name, "--gid", str(group_id)])
    if result.returncode != 0:
        message = i18n.t("install.create-group-failure")
        message = message.format(error=str(result.stderr))
        print_utils.fail(message)


def is_in_group(group_name):
    result = run(
        ["groups", _current_user()],
        stdout=subprocess.PIPE,
        encoding="utf-8",
        print_command=False,
    )
    return group_name in result.stdout.split()


def add_to_group(group_name):
    print_utils.translate("general.adding-to-group", group=group_name)
    run(["usermod", "-a", "-G", group_name, _current_user()])


def is_root():
    return os.geteuid() == 0


def give_brainframe_group_rw_access(paths: List[Path]):
    paths_str = [str(p) for p in paths]

    run(["chgrp", "-R", "brainframe"] + paths_str)
    run(["chmod", "-R", "g+rw"] + paths_str)


def _current_user():
    # If the SUDO_USER environment variable allows us to get the username of
    # the user running sudo instead of root. If they're not using sudo, we can
    # just pull the username from $LOGNAME.
    # "Why not use $USER here?" you might ask. Apparently $LOGNAME is a
    # POSIX standard and $USER is not.
    #  https://unix.stackexchange.com/a/76369/117461
    return os.environ.get("SUDO_USER", os.environ["LOGNAME"])


def run(
    command: List[str],
    print_command=True,
    exit_on_failure=True,
    *args,
    **kwargs,
) -> subprocess.CompletedProcess:
    """A small wrapper around subprocess.run.

    :param command: The command to run
    :param print_command: If True, the command will be printed before being run
    :param exit_on_failure: If True, the application will exit if the command
        results in a non-zero exit code
    """
    if print_command:
        print_utils.print_color(" ".join(command), print_utils.Color.MAGENTA)
    result = subprocess.run(command, *args, **kwargs)
    if result.returncode != 0 and exit_on_failure:
        sys.exit(result.returncode)
    return result


_SUPPORTED_DISTROS = {
    "Ubuntu": ["18.04"],
}
"""A dict whose keys are supported Linux distribution names and whose values
are all supported versions for that distribution.
"""


def is_supported() -> bool:
    """
    :return: True if the user is on an officially supported Linux distribution
    """
    name, version, _ = distro.linux_distribution()

    return name in _SUPPORTED_DISTROS and version in _SUPPORTED_DISTROS[name]



import sys

from brainframe.cli import docker_compose, env_vars
from .utils import command


@command("compose")
def compose():
    install_path = env_vars.install_path.get()
    docker_compose.assert_installed(install_path)
    docker_compose.run(install_path, sys.argv[2:])



import sys
from argparse import ArgumentParser
from datetime import datetime
from pathlib import Path

import i18n

from brainframe.cli import (
    print_utils,
    docker_compose,
    os_utils,
    dependencies,
    env_vars,
)
from .utils import subcommand_parse_args, command

BACKUP_DIR_FORMAT = "%Y-%m-%d_%H-%M-%S"


@command("backup")
def backup():
    install_path = env_vars.install_path.get()
    data_path = env_vars.data_path.get()

    args = _parse_args(data_path)

    # This command has to be run as root for now because some BrainFrame
    # services write files as the root user.
    if not os_utils.is_root():
        print_utils.fail_translate("general.user-not-root")

    docker_compose.assert_installed(install_path)

    dependencies.rsync.ensure(args.noninteractive, args.install_rsync)

    if not args.noninteractive:
        stop_brainframe = print_utils.ask_yes_no("backup.ask-stop-brainframe")
        if not stop_brainframe:
            # BrainFrame needs to be stopped before a backup. If the user
            # doesn't want that, stop the backup
            sys.exit(1)

    docker_compose.run(install_path, ["stop"])

    if args.destination is None:
        now_str = datetime.now().strftime(BACKUP_DIR_FORMAT)
        backup_path = data_path / "backups" / now_str
    else:
        backup_path = args.destination

    try:
        backup_path.mkdir(parents=True, exist_ok=True)
    except PermissionError:
        print_utils.fail_translate("backup.mkdir-permission-denied")

    os_utils.run(
        [
            "rsync",
            "--archive",
            "--verbose",
            "--progress",
            # Avoid backing up backups
            "--exclude",
            "backups",
            str(data_path),
            str(backup_path),
        ]
    )

    # Give the brainframe group access to the resulting backup, to make
    # managing and restoring it easier
    os_utils.give_brainframe_group_rw_access(backup_path)

    print()
    print_utils.translate("backup.complete", color=print_utils.Color.GREEN)


def _parse_args(data_path: Path):
    parser = ArgumentParser(
        description=i18n.t("backup.description"), usage=i18n.t("backup.usage")
    )

    parser.add_argument(
        "--destination",
        type=Path,
        help=i18n.t(
            "backup.destination-help", backup_dir=data_path / "backups"
        ),
    )

    parser.add_argument(
        "--noninteractive",
        action="store_true",
        help=i18n.t("general.noninteractive-help"),
    )

    parser.add_argument(
        "--install-rsync",
        action="store_true",
        help=i18n.t("backup.install-rsync-help"),
    )

    return subcommand_parse_args(parser)



import subprocess
from pathlib import Path
from typing import List

from . import os_utils, print_utils, env_vars

# The URL to the docker-compose.yml
BRAINFRAME_DOCKER_COMPOSE_URL = "https://{subdomain}aotu.ai/releases/brainframe/{version}/docker-compose.yml"
# The URL to the latest tag, which is just a file containing the latest version
# as a string
BRAINFRAME_LATEST_TAG_URL = (
    "https://{subdomain}aotu.ai/releases/brainframe/latest"
)


def assert_installed(install_path: Path):
    compose_path = install_path / "docker-compose.yml"

    if not compose_path.is_file():
        print_utils.fail_translate(
            "general.brainframe-must-be-installed",
            install_env_var=env_vars.install_path.name,
        )


def run(install_path: Path, commands: List[str]):
    compose_path = install_path / "docker-compose.yml"

    full_command = ["docker-compose", "-f", str(compose_path)]

    # Provide the override file if it exists
    compose_override_path = install_path / "docker-compose.override.yml"
    if compose_override_path.is_file():
        full_command += ["-f", str(compose_override_path)]

    os_utils.run(full_command + commands)


def download(target: Path, version="latest"):
    subdomain = ""
    auth_flags = []

    # Add the flags to authenticate with staging if the user wants to download
    # from there
    if env_vars.is_staging.is_set():
        subdomain = "staging."

        username = env_vars.staging_username.get()
        password = env_vars.staging_password.get()
        if username is None or password is None:
            print_utils.fail_translate(
                "general.staging-missing-credentials",
                username_env_var=env_vars.staging_username.name,
                password_env_var=env_vars.staging_password.name,
            )

        auth_flags = ["--user", f"{username}:{password}"]

    if version == "latest":
        # Check what the latest version is
        url = BRAINFRAME_LATEST_TAG_URL.format(subdomain=subdomain)
        result = os_utils.run(
            ["curl", "--fail", "--location", url] + auth_flags,
            stdout=subprocess.PIPE,
            encoding="utf-8",
        )
        version = result.stdout.strip()

    url = BRAINFRAME_DOCKER_COMPOSE_URL.format(
        subdomain=subdomain, version=version
    )
    os_utils.run(
        ["curl", "-o", str(target), "--fail", "--location", url] + auth_flags,
    )

    if os_utils.is_root():
        # Fix the permissions of the docker-compose.yml so that the BrainFrame
        # group can edit it
        os_utils.give_brainframe_group_rw_access([target])



import sys
from argparse import ArgumentParser

by_name = {}
"""A dict that maps command names to their corresponding function"""


def command(name):
    """A decorator that associates command functions to their command name."""

    def wrapper(function):
        by_name[name] = function

    return wrapper


def subcommand_parse_args(parser: ArgumentParser):
    arg_list = sys.argv[2:]
    args = parser.parse_args(arg_list)

    # Run in non-interactive mode if any flags were provided
    if len(arg_list) > 0:
        args.noninteractive = True

    return args



import os
from pathlib import Path
from typing import Optional, Generic, TypeVar, Callable, Union, Type

T = TypeVar("T")
"""The type of a configuration value"""


class EnvironmentVariable(Generic[T]):
    """Manages a configuration option using system environment variables"""

    def __init__(
        self,
        name,
        default: T = None,
        converter: Union[Type, Callable[[str], T]] = str,
    ):
        """
        :param name: The environment variable name
        :param default: The default value if the variable is unset
        :param converter: Converts the string value of the environment variable
            to the desired type
        """
        self.name = name
        self.default = default
        self.converter = converter

    def get(self) -> Optional[T]:
        if self.name in os.environ:
            value = os.environ[self.name]
            if value is None:
                return None
            return self.converter(value)

        return self.default

    def is_set(self) -> bool:
        return self.name in os.environ


install_path = EnvironmentVariable[Path](
    "BRAINFRAME_INSTALL_PATH",
    default=Path("/usr/local/share/brainframe/"),
    converter=Path,
)

data_path = EnvironmentVariable[Path](
    "BRAINFRAME_DATA_PATH",
    default=Path("/var/local/brainframe"),
    converter=Path,
)

is_staging = EnvironmentVariable[str]("BRAINFRAME_STAGING")
staging_username = EnvironmentVariable[str]("BRAINFRAME_STAGING_USERNAME")
staging_password = EnvironmentVariable[str]("BRAINFRAME_STAGING_PASSWORD")



import os
import signal
import sys
from argparse import ArgumentParser

import i18n

from brainframe.cli import (
    print_utils,
    commands,
    env_vars,
)


def main():
    i18n.load_path.append(_TRANSLATIONS_PATH)

    parser = ArgumentParser(
        description=i18n.t("portal.description"), usage=i18n.t("portal.usage")
    )

    parser.add_argument(
        "command", default=None, nargs="?", help=i18n.t("portal.command-help")
    )

    # This environment variable must be set as it is used by the
    # docker-compose.yml to find the data path to volume mount
    if not env_vars.data_path.is_set():
        os.environ[env_vars.data_path.name] = str(env_vars.data_path.default)

    args = parser.parse_args(sys.argv[1:2])

    # Exit with a clean error when interrupted
    def on_sigint(_sig, _frame):
        print()
        print_utils.fail_translate("portal.interrupted")

    signal.signal(signal.SIGINT, on_sigint)

    if args.command is None:
        print_utils.translate("portal.no-command-provided")
    elif args.command in commands.by_name:
        command = commands.by_name[args.command]
        command()
    else:
        error_message = i18n.t("portal.unknown-command")
        error_message = error_message.format(command=args.command)
        print_utils.print_color(
            error_message, color=print_utils.Color.RED, file=sys.stderr
        )
        parser.print_help()


_TRANSLATIONS_PATH = os.path.join(os.path.dirname(__file__), "translations")

if __name__ == "__main__":
    main()



import shutil

from . import print_utils, os_utils


class Dependency:
    """Provides functionality for discovering and installing an executable that
    the CLI depends on
    """

    def __init__(self, command_name, ask_message, install_func):
        self.command_name = command_name
        self.ask_message = ask_message
        self.install = install_func

    def ensure(self, noninteractive: bool, install_requested: bool):
        """Ensures that this dependency is installed, installing it if we're
        running on a supported OS and the user opts in. Exits with an error
        code if the dependency could not be ensured.

        :param noninteractive: If true, the user will not be prompted
        :param install_requested: If true, the user opted in to automatic
            noninteractive installation
        """
        # Only supported operating systems can request automatic installs
        if install_requested and not os_utils.is_supported():
            print_utils.fail_translate(
                "install.install-dependency-unsupported-os"
            )

        if shutil.which(self.command_name) is not None:
            # The command is already installed
            return

        print_utils.translate(
            "install.missing-dependency", dependency=self.command_name
        )

        if noninteractive:
            install = install_requested
        elif os_utils.is_supported():
            install = print_utils.ask_yes_no(self.ask_message)
        else:
            install = False

        if install:
            self.install()
        else:
            # The dependency is missing and we can't automatically install it.
            # Quit out with an error.
            print_utils.fail_translate(
                "install.install-dependency-manually",
                dependency=self.command_name,
            )


def _install_docker():
    os_utils.run(
        ["curl", "-fsSL", "https://get.docker.com", "-o", "/tmp/get-docker.sh"]
    )
    os_utils.run(["sh", "/tmp/get-docker.sh"])


docker = Dependency("docker", "install.ask-install-docker", _install_docker,)

docker_compose = Dependency(
    "docker-compose",
    "install.ask-install-docker-compose",
    lambda: os_utils.run(["pip3", "install", "docker-compose"]),
)

rsync = Dependency(
    "rsync",
    "install.ask-install-rsync",
    lambda: os_utils.run(["apt-get", "install", "-y", "rsync"]),
)

curl = Dependency(
    "curl",
    "install.ask-install-curl",
    lambda: os_utils.run(["apt-get", "install", "-y", "curl"]),
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['commands']

package_data = \
{'': ['*']}

install_requires = \
['distro>=1.5,<2.0', 'python-i18n>=0.3,<0.4', 'pyyaml>=5.3,<6.0']

entry_points = \
{'console_scripts': ['brainframe = brainframe.cli.main:main']}

setup_kwargs = {
    'name': 'brainframe-cli',
    'version': '0.1.0.dev12',
    'description': 'Makes installing and managing a BrainFrame server easy',
    'long_description': '',
    'author': 'Aotu',
    'author_email': 'info@aotu.ai',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/aotuai/brainframe_cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)



import sys
from enum import Enum
from pathlib import Path
import os

import i18n


class Color(Enum):
    """ANSI escape codes representing colors in the terminal theme."""

    MAGENTA = "\033[95m"
    BLUE = "\033[94m"
    GREEN = "\033[92m"
    YELLOW = "\033[93m"
    RED = "\033[91m"
    END = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"


def ask_yes_no(message_id) -> bool:
    while True:
        choice = input_color(f"{i18n.t(message_id)} [y/n] ", Color.BLUE)
        choice = choice.strip().lower()
        if choice not in ["y", "n"]:
            translate("general.invalid-yes-no-input")
            continue

        print()

        return choice == "y"


def ask_path(message_id, default: Path) -> Path:
    message = i18n.t(message_id) + "\n"
    message += f"[{default}] "
    choice = input_color(message, Color.BLUE)

    print()

    if len(choice.strip()) == 0:
        return default

    return Path(choice)


def art() -> None:
    print_color(_BRAINFRAME_ART, Color.MAGENTA)


def translate(message_id, color=Color.END, **kwargs):
    print_color(i18n.t(message_id, **kwargs), color)


def warning_translate(message_id, color=Color.YELLOW, **kwargs):
    print_color(i18n.t(message_id), color, **kwargs)


def fail_translate(message_id, **kwargs):
    fail(i18n.t(message_id, **kwargs))


def fail(message, **kwargs):
    print_color(message, Color.RED, file=sys.stderr, **kwargs)
    sys.exit(1)


def print_color(message, color: Color, **kwargs) -> None:
    color = _check_no_color(color)
    print(f"{color.value}{message}{Color.END.value}", **kwargs)


def input_color(message, color: Color) -> str:
    color = _check_no_color(color)
    return input(f"{color.value}{message}{Color.END.value}")


def _check_no_color(color: Color) -> Color:
    """Turns off color if the user wants.
    See: https://no-color.org/
    """
    if "NO_COLOR" in os.environ:
        return Color.END
    return color


_DOCKER_COMPOSE_DOWNLOAD_URL = (
    "https://github.com/docker/compose/releases/download/1.25.5/"
    "docker-compose-Linux-x86_64"
)


_BRAINFRAME_ART = r"""
 _______            __       _______
|   _   .----.---.-|__.-----|   _   .----.---.-.--------.-----.
|.  1   |   _|  _  |  |     |.  1___|   _|  _  |        |  -__|
|.  _   |__| |___._|__|__|__|.  __) |__| |___._|__|__|__|_____|
|:  1    \                  |:  |
|::.. .  /                  |::.|
`-------'                   `---'
                                        Installer

"""



from argparse import ArgumentParser
from pathlib import Path

import i18n

from brainframe.cli import (
    print_utils,
    docker_compose,
    env_vars,
    os_utils,
    dependencies,
)
from .utils import subcommand_parse_args, command


@command("install")
def install():
    args = _parse_args()

    if not os_utils.is_root():
        print_utils.fail_translate("general.user-not-root")

    # Print some introductory text
    if not args.noninteractive:
        print_utils.art()
        print_utils.translate("install.thanks")
        print()
    if not os_utils.is_supported():
        print_utils.warning_translate("install.unsupported-os")
        print()

    # Check all dependencies
    dependencies.curl.ensure(args.noninteractive, args.install_curl)
    dependencies.docker.ensure(args.noninteractive, args.install_docker)
    dependencies.docker_compose.ensure(
        args.noninteractive, args.install_docker_compose
    )

    added_to_any_group = False

    if not os_utils.is_in_group("docker"):
        if args.noninteractive:
            add_to_group = args.add_to_docker_group
        else:
            add_to_group = print_utils.ask_yes_no(
                "install.ask-add-to-docker-group"
            )

        if add_to_group:
            os_utils.add_to_group("docker")
            added_to_any_group = True

    # Set up the install path
    if args.noninteractive:
        install_path = args.install_path
    else:
        install_path = print_utils.ask_path(
            "install.ask-brainframe-install-path",
            env_vars.install_path.default,
        )
    install_path.mkdir(exist_ok=True, parents=True)

    # Set up the data path
    if args.noninteractive:
        data_path = args.data_path
    else:
        data_path = print_utils.ask_path(
            "install.ask-data-path", env_vars.data_path.default
        )
    data_path.mkdir(exist_ok=True, parents=True)

    # Set up permissions with the 'brainframe' group
    print_utils.translate("install.create-group-justification")
    os_utils.create_group("brainframe", os_utils.BRAINFRAME_GROUP_ID)
    os_utils.give_brainframe_group_rw_access([data_path, install_path])

    # Optionally add the user to the "brainframe" group
    if not os_utils.is_in_group("brainframe"):
        if args.noninteractive:
            add_to_group = args.add_to_group
        else:
            add_to_group = print_utils.ask_yes_no("install.ask-add-to-group")

        if add_to_group:
            os_utils.add_to_group("brainframe")
            added_to_any_group = True

    docker_compose.download(
        install_path / "docker-compose.yml", version=args.version
    )

    print_utils.translate("install.downloading-images")
    docker_compose.run(install_path, ["pull"])

    if not args.noninteractive and print_utils.ask_yes_no("install.ask-start"):
        docker_compose.run(install_path, ["up", "-d"])
    else:
        print_utils.translate("install.how-to-start")

    # Recommend to the user to add their custom paths to environment variables
    # so that future invocations of the program will know where to look.
    if (
        install_path != env_vars.install_path.default
        or data_path != env_vars.data_path.default
    ):
        print()
        print_utils.translate("install.set-custom-directory-env-vars")
        print(
            f"\n"
            f'export {env_vars.install_path.name}="{install_path}"\n'
            f'export {env_vars.data_path.name}="{data_path}"\n'
        )

    if added_to_any_group:
        print_utils.translate("install.logout-to-apply-group-changes")

    print()
    print_utils.translate("install.complete", print_utils.Color.GREEN)


def _parse_args():
    parser = ArgumentParser(
        description=i18n.t("install.description"),
        usage=i18n.t("install.usage"),
    )

    parser.add_argument(
        "--noninteractive",
        action="store_true",
        help=i18n.t("general.noninteractive-help"),
    )
    parser.add_argument(
        "--install-path",
        type=Path,
        default=env_vars.install_path.default,
        help=i18n.t("install.install-path-help"),
    )
    parser.add_argument(
        "--data-path",
        type=Path,
        default=env_vars.data_path.default,
        help=i18n.t("install.data-path-help"),
    )
    parser.add_argument(
        "--install-docker",
        action="store_true",
        help=i18n.t("install.install-docker-help"),
    )
    parser.add_argument(
        "--install-docker-compose",
        action="store_true",
        help=i18n.t("install.install-docker-compose-help"),
    )
    parser.add_argument(
        "--install-curl",
        action="store_true",
        help=i18n.t("install.install-curl-help"),
    )
    parser.add_argument(
        "--add-to-group",
        action="store_true",
        help=i18n.t("install.add-to-group-help"),
    )
    parser.add_argument(
        "--add-to-docker-group",
        action="store_true",
        help=i18n.t("install.add-to-docker-group-help"),
    )
    parser.add_argument(
        "--version",
        type=str,
        default="latest",
        help=i18n.t("install.version-help"),
    )

    return subcommand_parse_args(parser)
